#!/bin/bash

ROOT_UUID=$(sed -e 's/^.*root=//' -e 's/ .*$//' < /proc/cmdline)
ROOT_PARTITION=$(blkid | tr -d '":' | grep "${ROOT_UUID}" | awk '{print $1}')
ROOT_DEVICE=/dev/$(lsblk -ndo pkname ${ROOT_PARTITION})
EMMC_DEVICES=$(ls -d -1 /dev/mmcblk* 2>/dev/null | grep -w 'mmcblk[0-9]' | grep -v "${ROOT_DEVICE}")
echo "Устройство, откуда будет производиться копирование: ${ROOT_DEVICE}"
echo "Доступные устройства для которых можно произвести копирование: ${EMMC_DEVICES}"

# Преобразование списка доступных устройств в массив
EMMC_DEVICES_ARRAY=($EMMC_DEVICES)

# Проверка на пустоту массива устройств
if [ ${#EMMC_DEVICES_ARRAY[@]} -eq 0 ]; then
    whiptail \
      --title "Невозможно произвести копирование" \
      --msgbox "Не найдено ни одного устройства eMMC." \
      8 45
    exit 0
fi

# Создание строки для whiptail
MENU_ITEMS=()
for DEVICE in "${EMMC_DEVICES_ARRAY[@]}"; do
    MENU_ITEMS+=("$DEVICE" "")
done

# Отображение меню с помощью whiptail
EMMC_DEVICE=$(
  whiptail \
    --title "Выбор устройства" \
    --menu "Копирование будет производиться из ${ROOT_DEVICE}\nВыберите устройство для копирования:" \
    16 60 4 \
    "${MENU_ITEMS[@]}" \
    3>&1 1>&2 2>&3)

# Проверка кода возврата
EXIT_STATUS=$?
if [ $EXIT_STATUS -eq 1 ] || [ $EXIT_STATUS -eq 255 ]; then
  exit 0
fi

# Получение размеров устройств
ROOT_SIZE=$(lsblk -bno SIZE "${ROOT_DEVICE}p1")
EMMC_SIZE=$(lsblk -bno SIZE "${EMMC_DEVICE}p1")

# Проверка размеров устройств
if [ "$EMMC_SIZE" -lt "$ROOT_SIZE" ]; then
    whiptail \
      --title "Ошибка" \
      --msgbox "Размер устройства eMMC (${EMMC_SIZE} байт) меньше, чем размер ROOT устройства (${ROOT_SIZE} байт)." \
      8 45
    exit 0
fi

# Полное форматирование eMMC
echo "Полное форматирование eMMC (${EMMC_DEVICE})..."
blkdiscard ${EMMC_DEVICE}

# Удаление всех разделов eMMC
echo "Удаление всех разделов eMMC (${EMMC_DEVICE})..."
dd bs=1 seek=446 count=64 if=/dev/zero of="${EMMC_DEVICE}" >/dev/null 2>&1

# Подсчет размера eMMC
echo "Подсчет размера eMMC..."
QUOTED_DEVICE=$(echo "${EMMC_DEVICE}" | sed 's:/:\\\/:g')
CAPACITY=$(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", \$2 / ( 1024 / \$4 ))}")
echo "Объем eMMC: ${CAPACITY}"

FIRSTSECTOR=32768
# Подсчет размера раздела boot
if [[ $CAPACITY -lt 4000000 ]]; then
		# Leave 2 percent unpartitioned when eMMC size is less than 4GB (unlikely)
		LASTSECTOR=$(( 32768 * $(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", ( \$2 * 98 / 3276800))}") -1 ))
	else
		# Leave 1 percent unpartitioned
		LASTSECTOR=$(( 32768 * $(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", ( \$2 * 99 / 3276800))}") -1 ))
fi
echo "Первый сектор на eMMC: ${FIRSTSECTOR}"
echo "Последний сектор на eMMC: ${LASTSECTOR}"

PART_TABLE_TYPE=$(parted "${ROOT_DEVICE}" print -sm | awk -F ":" -v pattern="${ROOT_DEVICE}" '$0 ~ pattern {print $6}')
echo "Раздел ${ROOT_DEVICE} имеет тип файловой системы: ${PART_TABLE_TYPE}"

parted -s "${EMMC_DEVICE}" -- mklabel "$PART_TABLE_TYPE"

parted -s "${EMMC_DEVICE}" -- mkpart primary ext4 ${FIRSTSECTOR}s ${LASTSECTOR}s
partprobe "${EMMC_DEVICE}"
mkfs.ext4 -qF "${EMMC_DEVICE}"'p1'
emmcuuid=$(blkid -o export "${EMMC_DEVICE}"'p1' | grep -w UUID)
emmcbootuuid=$emmcuuid
echo "Идентификатор eMMC: ${emmcuuid}"

echo "Запись загрузчика на eMMC..."
dd if=/dev/zero of=${EMMC_DEVICE} bs=1k count=1023 seek=1 status=noxfer > /dev/null 2>&1

echo "Определение текущего загрузчика"
BOOT_MESSAGE=$(repka-control get-boot-message)
if [ ${BOOT_MESSAGE} = '0' ]; then
  echo "Режим загрузчика: тихий режим"
  dd if=/boot/spl/u-boot-sunxi-with-spl-silent.bin of=${EMMC_DEVICE} bs=1024 seek=8 status=noxfer > /dev/null 2>&1
else
  echo "Режим загрузчика: стандартный режим"
  dd if=/boot/spl/u-boot-sunxi-with-spl.bin of=${EMMC_DEVICE} bs=1024 seek=8 status=noxfer > /dev/null 2>&1
fi


echo "Создание точек монтирования..."
mkdir /mnt/root /mnt/emmc
mount "${ROOT_DEVICE}p1" /mnt/root
mount "${EMMC_DEVICE}p1" /mnt/emmc

echo "Копирование содержимого корневой файловой системы..."
rsync -aAXv --info=progress2 --exclude='/mnt/*' --exclude='/run/*' --exclude='/tmp/*' --exclude='/var/tmp/*' --exclude='/var/run/*' --exclude='/sys/*' --exclude='/proc/*' --exclude='/dev/*' /mnt/root/ /mnt/emmc/

echo "Удаление точек монтирования и их содержимого..."
umount /mnt/root /mnt/emmc
rmdir /mnt/root /mnt/emmc

whiptail --title "OS Copying" --msgbox "Произошло удачное клонирование Repka OS с ${ROOT_DEVICE} на ${EMMC_DEVICE}" 8 45